// local header files
#include <ColorPointWidget.h>
#include <GradientOpacityWidget.h>
#include <TransparencyPointWidget.h>
#include <VolumeRenderingAction.h>
#include <VolumeRenderingExtension.h>
#include <VolumeRenderingWidget.h>

