/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/

#ifndef STLCOMPONENT_H
#define STLCOMPONENT_H

#include <QObject>

#include <MeshComponent.h>

/**
 *  @ingroup group_sdk_components_stl
 *
 *  @brief
 *  The manager of the STL format, see https://en.wikipedia.org/wiki/STL_%28file_format%29.
 *  Simply use vtkSTLReader to build a MeshComponent
 *
 **/
class StlComponent : public camitk::MeshComponent {

    Q_OBJECT

public:
    /// Default Constructor
    /// This method may throw an AbortException if a problem occurs.
    StlComponent(const QString& file);

    /// Default Destructor
    virtual ~StlComponent() = default;

};

#endif // STLCOMPONENT_H

