/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2018 Univ. Grenoble Alpes, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/

#ifndef MMLMonitorDisplay_H
#define MMLMonitorDisplay_H

// Monitoring includes
#include <Monitor.h>

#include <QObject>
#include "MMLComponentAPI.h"

class MMLComponent;

/**
 * @ingroup group_cepmodeling_components_mml
 *
 * @brief
 * Generic class for MML Monitor 3D/Graphical representation
 *
 *
 **/
class MMLCOMPONENT_API MMLMonitorDisplay : public QObject {


public:

    /// constructor
    MMLMonitorDisplay(Monitor* monitor, MMLComponent* manager);
    /// destructor
    ~MMLMonitorDisplay() override = default;

    /// update display
    virtual void update() = 0;
    /// hide display
    virtual void hide() = 0;

protected:
    /// monitor
    Monitor* monitor;
    /// display
    MMLComponent* manager;
};

#endif
